
/* 
 * Copyright (c) 1993-2003,2012 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*------------------  Fichier gnr automatiquement ------------------*/
/*------------------  Ne pas diter manuellement !!! ------------------*/

#ifndef pilo_MSG_LIB_H
#define pilo_MSG_LIB_H

#include "portLib.h"
#include "csLib.h"
#include "posterLib.h"
 
#include "piloType.h"

#define TIME_WAIT_REPLY           /*2000*/ 0
 



/*----------- Tailles des messages et boites aux lettres  ----------*/

/* Taille du buffer utilise dans la bal pour un mesage de taille size
 * (cf servMbox pour UNIX et h2rgnLib pour VXWORKS )
 */


#define BUF_SIZE(size) \
   (size + 4 - (size & 3) + sizeof(LETTER_HDR) + 8)

/**
 ** Boite aux lettres reception requetes 
 **/
#define  PILO_MBOX_NAME                  genomInstanceName("pilo")

#define  PILO_MAX_RQST_SIZE              1664

#define  PILO_MBOX_RQST_SIZE       \
 (BUF_SIZE(PILO_MAX_RQST_SIZE) * SERV_NMAX_RQST_ID)

/** 
 ** Taille max repliques (serveur)
 **/
#define  PILO_MAX_INTERMED_REPLY_SIZE   (sizeof(int))

#define  PILO_MAX_REPLY_SIZE             16

/** 
 ** Taille bal clients pour reception repliques 
 **/
#define  PILO_CLIENT_MBOX_REPLY_SIZE   \
((BUF_SIZE(PILO_MAX_REPLY_SIZE) \
   + BUF_SIZE(PILO_MAX_INTERMED_REPLY_SIZE)) \
 * CLIENT_NMAX_RQST_ID)

/*-------------------------- Liste des requetes  -------------------------*/

#define PILO_INIT_RQST 0
#define PILO_MOVE_RQST 1
#define PILO_TURN_RQST 2
#define PILO_GOTOCONFIG_RQST 3
#define PILO_SETDYNAMIC_RQST 4
#define PILO_GETDYNAMIC_RQST 5
#define PILO_EXECTRAJ_RQST 6
#define PILO_EXECPOSTERTRAJ_RQST 7
#define PILO_EXECABSOLUTEBL_RQST 8
#define PILO_STOP_RQST 9
#define PILO_SLOW_RQST 10
#define PILO_MONLENGTH_RQST 11
#define PILO_ABORT_RQST 12


/*---------------- PROTOTYPES DES FONCTIONS EXTERNES ------------------*/

#ifdef __cplusplus
extern "C" {
#endif

STATUS piloClientInit (CLIENT_ID *pClientId);

STATUS piloClientEnd (CLIENT_ID clientId);

int piloAbortRqstSend (CLIENT_ID clientId, 
			   int *pRqstId,
			   int *activity,
			   int replyTimeOut);

int piloAbortReplyRcv (CLIENT_ID clientId, 
			   int rqstId, 
			   int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			   int *bilan);

int piloAbortRqstAndRcv (CLIENT_ID clientId, 
			     int *activity,
			     int *bilan);




extern STATUS piloInitRqstSend (CLIENT_ID clientId, int *pRqstId,
					 PILO_INIT_STR *(in_initStr),
					 int replyTimeOut);

extern int piloInitReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int piloInitRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					PILO_INIT_STR *(in_initStr), 
					
					int *activity, int *bilan);

extern int piloInitRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					PILO_INIT_STR *(in_initStr), 
					
					int *activity,
					int *bilan);




extern STATUS piloMoveRqstSend (CLIENT_ID clientId, int *pRqstId,
					 PILO_MOVE_STR *(in_moveStr),
					 int replyTimeOut);

extern int piloMoveReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int piloMoveRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					PILO_MOVE_STR *(in_moveStr), 
					
					int *activity, int *bilan);

extern int piloMoveRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					PILO_MOVE_STR *(in_moveStr), 
					
					int *activity,
					int *bilan);




extern STATUS piloTurnRqstSend (CLIENT_ID clientId, int *pRqstId,
					 PILO_TURN_STR *(in_turnStr),
					 int replyTimeOut);

extern int piloTurnReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int piloTurnRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					PILO_TURN_STR *(in_turnStr), 
					
					int *activity, int *bilan);

extern int piloTurnRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					PILO_TURN_STR *(in_turnStr), 
					
					int *activity,
					int *bilan);




extern STATUS piloGotoConfigRqstSend (CLIENT_ID clientId, int *pRqstId,
					 PILO_GOTO_CONFIG_STR *(in_gotoStr),
					 int replyTimeOut);

extern int piloGotoConfigReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int piloGotoConfigRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					PILO_GOTO_CONFIG_STR *(in_gotoStr), 
					
					int *activity, int *bilan);

extern int piloGotoConfigRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					PILO_GOTO_CONFIG_STR *(in_gotoStr), 
					
					int *activity,
					int *bilan);




extern STATUS piloSetDynamicRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 PILO_DYNAMIC_PARAMS *(in_dynamicParams),
					 int replyTimeOut);

int piloSetDynamicReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int piloSetDynamicRqstAndRcv (CLIENT_ID clientId, 
				 PILO_DYNAMIC_PARAMS *(in_dynamicParams), 
				 
				 int *bilan);




extern STATUS piloGetDynamicRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int piloGetDynamicReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       PILO_DYNAMIC_PARAMS *(out_dynamicParams),
			       int *bilan);

int piloGetDynamicRqstAndRcv (CLIENT_ID clientId, 
				  
				 PILO_DYNAMIC_PARAMS *(out_dynamicParams),
				 int *bilan);




extern STATUS piloExecTrajRqstSend (CLIENT_ID clientId, int *pRqstId,
					 PILO_TRAJECTORY *(in_trajStr),
					 int replyTimeOut);

extern int piloExecTrajReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int piloExecTrajRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					PILO_TRAJECTORY *(in_trajStr), 
					
					int *activity, int *bilan);

extern int piloExecTrajRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					PILO_TRAJECTORY *(in_trajStr), 
					
					int *activity,
					int *bilan);




extern STATUS piloExecPosterTrajRqstSend (CLIENT_ID clientId, int *pRqstId,
					 PILO_TRAJ_POSTER *(in_trajPoster),
					 int replyTimeOut);

extern int piloExecPosterTrajReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int piloExecPosterTrajRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					PILO_TRAJ_POSTER *(in_trajPoster), 
					
					int *activity, int *bilan);

extern int piloExecPosterTrajRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					PILO_TRAJ_POSTER *(in_trajPoster), 
					
					int *activity,
					int *bilan);




extern STATUS piloExecAbsoluteBLRqstSend (CLIENT_ID clientId, int *pRqstId,
					 PILO_ABSOLUTE_BL *(in_absBLStr),
					 int replyTimeOut);

extern int piloExecAbsoluteBLReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int piloExecAbsoluteBLRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					PILO_ABSOLUTE_BL *(in_absBLStr), 
					
					int *activity, int *bilan);

extern int piloExecAbsoluteBLRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					PILO_ABSOLUTE_BL *(in_absBLStr), 
					
					int *activity,
					int *bilan);




extern STATUS piloStopRqstSend (CLIENT_ID clientId, int *pRqstId,
					 
					 int replyTimeOut);

extern int piloStopReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int piloStopRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					 
					
					int *activity, int *bilan);

extern int piloStopRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					 
					
					int *activity,
					int *bilan);




extern STATUS piloSlowRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 int *(in_percentSpeed),
					 int replyTimeOut);

int piloSlowReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int piloSlowRqstAndRcv (CLIENT_ID clientId, 
				 int *(in_percentSpeed), 
				 
				 int *bilan);




extern STATUS piloMonLengthRqstSend (CLIENT_ID clientId, int *pRqstId,
					 double *(in_monLength),
					 int replyTimeOut);

extern int piloMonLengthReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int piloMonLengthRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					double *(in_monLength), 
					
					int *activity, int *bilan);

extern int piloMonLengthRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					double *(in_monLength), 
					
					int *activity,
					int *bilan);


#ifdef __cplusplus
}
#endif
#endif /* pilo_MSG_LIB_H */

